--------------------------------------------------------------------------------
-- Nyan Cat by holzchopf
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.nyancat={}
holzchopf.nyancat.cat={}
holzchopf.nyancat.rainbow={
	{red=255,grn=0,blu=0},
	{red=255,grn=153,blu=0},
	{red=255,grn=255,blu=0},
	{red=51,grn=255,blu=0},
	{red=0,grn=153,blu=255},
	{red=102,grn=51,blu=255}
}
holzchopf.nyancat.trail={}
holzchopf.nyancat.traillength=0

-- load ressources
holzchopf.nyancat.gfx_wpn={}
for i=1,7 do
	holzchopf.nyancat.gfx_wpn[i]=loadgfx("holzchopf/nyancat"..i..".png")
	setmidhandle(holzchopf.nyancat.gfx_wpn[i])
end
holzchopf.nyancat.gfx_trail=loadgfx("holzchopf/nyancattrail.png")
setmidhandle(holzchopf.nyancat.gfx_trail)
holzchopf.nyancat.gfx_ring=loadgfx("holzchopf/nyancatring.png")
setmidhandle(holzchopf.nyancat.gfx_ring)
holzchopf.nyancat.gfx_part=loadgfx("holzchopf/nyancatparticle.png")
setmidhandle(holzchopf.nyancat.gfx_part)
holzchopf.nyancat.sfx_music=loadsfx("holzchopf/Nyanyanya (Loop).ogg")

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.nyancat.id=addweapon("holzchopf.nyancat","Nyan Cat",holzchopf.nyancat.gfx_wpn[1],0,3)

function holzchopf.nyancat.draw()
	if weapon_shots==0 then
		if getplayerdirection(0)==1 then
			hudinfo("Hit [attack] to launch Nyan Cat in the west end of the map!")
		else
			hudinfo("Hit [attack] to launch Nyan Cat in the east end of the map!")
		end
	end
end

function holzchopf.nyancat.attack(attack)
	if attack==1 and weapon_shots==0 then
		-- use weapon
		useweapon(0)
		playercontrol(0)
		weapon_shots=weapon_shots+1
		holzchopf.nyancat.trail={}
		local secondsleft=getframesleft()/50
		changeturntime(20-secondsleft)
		-- create projectile
		id=createprojectile(holzchopf.nyancat.cat.id)
		projectiles[id]={}
		projectiles[id].timer=getframe()+5
		projectiles[id].frame=1
		projectiles[id].path=0
		projectiles[id].mode=1
		-- pose
		if getplayerdirection(0)==-1 then
			projectiles[id].x=getmapwidth()+800
		else
			projectiles[id].x=-800
		end
		projectiles[id].y=getmapheight()/2
		projectiles[id].dir=getplayerdirection(0)
		-- speed
		projectiles[id].sx=getplayerdirection(0)*6
		projectiles[id].sy=0
		-- draw initial trail
		for i=1,200 do
			local f=i/2
			local dy=0.5+math.sin(projectiles[id].path)
			projectiles[id].path=projectiles[id].path+1
			local newtrail={}
			newtrail.rot=90*projectiles[id].dir
			newtrail.x=projectiles[id].x-projectiles[id].dir*9-i*3*projectiles[id].dir
			newtrail.y=projectiles[id].y+dy*projectiles[id].dir
			newtrail.timer=getframe()-f*0.1
			holzchopf.nyancat.trail[#holzchopf.nyancat.trail+1]=newtrail
		end
	end
end

--------------------------------------------------------------------------------
-- Cat
--------------------------------------------------------------------------------
holzchopf.nyancat.cat.id=addprojectile("holzchopf.nyancat.cat")

function holzchopf.nyancat.cat.draw(id)
	-- draw trail
	setcolor(255,255,255)
	setblend(blend_alpha)
	setalpha(1)
	setscale(projectiles[id].dir,1)
	holzchopf.nyancat.traillength=0
	for k,trail in pairs(holzchopf.nyancat.trail) do
		local alpha=1-(getframe()-trail.timer)*0.01
		if alpha>0 then
			setalpha(alpha)
			setrotation(trail.rot)
			drawimage(holzchopf.nyancat.gfx_trail,trail.x,trail.y)
			holzchopf.nyancat.traillength=holzchopf.nyancat.traillength+1
		else
			trail=nil
		end
	end
	-- draw cat
	if projectiles[id].mode~=0 then
		-- inform player how to steer
		hudinfo("Steer Nyan Cat with [left] and [right]")
		local rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
		setrotation(rot)
		drawimage(holzchopf.nyancat.gfx_wpn[projectiles[id].frame],projectiles[id].x,projectiles[id].y)
		-- scroll to cat
		scroll(projectiles[id].x,projectiles[id].y)
	-- draw explosion
	elseif projectiles[id].expltimer~=nil then
		local t=(getframe()-projectiles[id].expltimer)*0.01
		setalpha(1-t^2)
		setblend(blend_light)
		for i=1,30 do
			local f=(i/30)^2
			local a=f*12*(getframe()/100)
			local r=50-f*49
			local s=2-f*1.5
			setscale(s,s)
			setcolor(holzchopf.nyancat.rainbow[i%6+1].red,
			 holzchopf.nyancat.rainbow[i%6+1].grn,
			 holzchopf.nyancat.rainbow[i%6+1].blu)
			drawimage(holzchopf.nyancat.gfx_part,projectiles[id].x+math.cos(a)*r,
			 projectiles[id].y+math.sin(a)*r)
		end
		if t<0.4 then
			setalpha(1-t/0.4)
			setcolor(255,255,255)
			setscale(0.2+t*10,0.2+t*10)
			drawimage(holzchopf.nyancat.gfx_ring,projectiles[id].x,projectiles[id].y)
		end
	end
end

function holzchopf.nyancat.cat.update(id)
	if projectiles[id].mode==0 then
		if holzchopf.nyancat.traillength==0 then
			freeprojectile(id)
			endturn()
		end
		return
	end
	-- change frame
	if getframe()>=projectiles[id].timer then
		projectiles[id].timer=getframe()+4
		projectiles[id].frame=(projectiles[id].frame%7)+1
	end
	-- remove when time is over
	if getframesleft()<=0 then
		holzchopf.nyancat.cat.explode(id)
		return
	end
	-- steer
	local rot_s=math.atan2(projectiles[id].sx,-projectiles[id].sy)
	local rot_e=rot_s+(keydown(key_right)-keydown(key_left))*0.1
	local px=math.sin(rot_e)
	local py=-math.cos(rot_e)
	projectiles[id].sx=px*6
	projectiles[id].sy=py*6
	-- start pos of trail
	local x_s=projectiles[id].x-12*math.sin(rot_s)
	local y_s=projectiles[id].y+12*math.cos(rot_s)
	-- move (in substep loop for optimal collision precision)
	local msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/5)
	local msubx=projectiles[id].sx/msubt
	local msuby=projectiles[id].sy/msubt
	for i=1,msubt do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- collision
		if collision(col10x10,projectiles[id].x,projectiles[id].y,1,0,0)==1 then
			holzchopf.nyancat.cat.explode(id)
			return
		end
		-- water
		if (projectiles[id].y+20)>getwatery() then
			-- fx
			stopchannel(0)
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- free projectile
			projectiles[id].mode=0
			return
		end
	end
	-- check distance to players
	local players=playertable()
	for i=1,#players do
		if getplayeraction(players[i])==0 then
			local x=getplayerx(players[i])
			local y=getplayery(players[i])
			local dx=x-projectiles[id].x
			local dy=y-projectiles[id].y
			local dist=math.sqrt(dx*dx+dy*dy)
			-- let close players jump away
			if dist<=50 then
				local sgn=1
				if dx<0 then sgn=-1 end
				playerpush(players[i],0.3*(50-dist*0.7)*sgn,-0.2*(50-dist*0.7),1,0)
			end
		end
	end
	-- sfx
	if channelplaying(0)==0 then playsound(holzchopf.nyancat.sfx_music,0) end
	-- end pos of trail
	local x_e=projectiles[id].x-12*math.sin(rot_e)
	local y_e=projectiles[id].y+12*math.cos(rot_e)
	-- gfx
	local dy=math.sin(getframe())*2
	for i=1,2 do
		local f=i/2
		local dy=0.5+math.sin(projectiles[id].path)
		projectiles[id].path=projectiles[id].path+1
		local rot=rot_s+(rot_e-rot_s)*f
		local newtrail={}
		newtrail.rot=math.deg(rot)
		newtrail.x=x_s+(x_e-x_s)*f+dy*math.cos(rot)
		newtrail.y=y_s+(y_e-y_s)*f+dy*math.sin(rot)
		newtrail.timer=getframe()+f*0.1
		holzchopf.nyancat.trail[#holzchopf.nyancat.trail+1]=newtrail
	end
end

function holzchopf.nyancat.cat.explode(id)
	local x=projectiles[id].x
	local y=projectiles[id].y
	-- damage
	arealdamage(x,y,125,65)
	quake(20)
	-- destroy terrain
	terraincircle(x,y,55,0x00000000)
	-- crater
	for c=0,11 do
		local a=math.pi/6*c
		terrainalphaimage(gfx_crater50,x+math.cos(a)*50,y+math.sin(a)*50,0.8,
		 holzchopf.nyancat.rainbow[c%6+1].red,
		 holzchopf.nyancat.rainbow[c%6+1].grn,
		 holzchopf.nyancat.rainbow[c%6+1].blu)
	end
	-- gfx
	-- local c=math.random(1,6)
	-- for i=1,10 do
		-- particle(p_smoke,x,y)
		-- particlecolor(holzchopf.nyancat.rainbow[c].red,
		 -- holzchopf.nyancat.rainbow[c].grn,
		 -- holzchopf.nyancat.rainbow[c].blu)
		-- particlesize(80,80)
		-- particlealpha(1)
		-- particlefadealpha(0.01)
		-- particlespeed(0,0)
	-- end
	-- for i=1,6 do
		-- -- coloured explosion
		-- particle(p_explosion,x,y)
		-- particlecolor(holzchopf.nyancat.rainbow[i].red,
		 -- holzchopf.nyancat.rainbow[i].grn,
		 -- holzchopf.nyancat.rainbow[i].blu)
		-- particlesize(1,1)
		-- particlealpha(1)
		-- particlefadealpha(0.01)
		-- particlespeed(math.random(-10,10)*0.05,math.random(-10,10)*0.05)
		-- -- rings
		-- particle(p_ring,x,y)
		-- particlecolor(holzchopf.nyancat.rainbow[i].red,
		 -- holzchopf.nyancat.rainbow[i].grn,
		 -- holzchopf.nyancat.rainbow[i].blu)
		-- particlesize(0.4-i*0.03,0.4-i*0.03)
		-- particlefadealpha(0.04)
		-- particlespeed(1.07,1.07)
	-- end
	-- sparkles
	for i=0,10 do
		local a=math.rad(math.random(1,360))
		local s=math.random(5,10)*0.4
		local c=math.random(1,6)
		for j=1,4 do
			particle(p_flare,x,y)
			local aa=a+math.random()*0.2
			local ss=s+math.random()*0.5
			particlespeed(math.cos(aa)*ss,math.sin(aa)*ss)
			particlecolor(holzchopf.nyancat.rainbow[c].red,
			 holzchopf.nyancat.rainbow[c].grn,
			 holzchopf.nyancat.rainbow[c].blu)
			particlefadealpha(0.02)
			particlesize(10,10)
		end
	end
	-- "free" projectile
	stopchannel(0)
	projectiles[id].mode=0
	projectiles[id].expltimer=getframe()
	-- sfx
	playsound(_G["sfx_explosion"..math.random(1,7)])
end
